@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-import os, sys, stat, time
+import sys, stat, time
 from bup import options, git, index, drecurse
 from bup.helpers import *
 
@@ -152,7 +152,7 @@ if opt.check:
 paths = index.reduce_paths(extra)
 
 if opt.update:
-    if not paths:
+    if not extra:
         o.fatal('update (-u) requested but no paths given')
     for (rp,path) in paths:
         update_index(rp)
