@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-import sys, os, stat
+import sys, stat
 from bup import options, git, vfs
 from bup.helpers import *
 
