@@ -1,12 +1,12 @@
 #!/usr/bin/env python
-import sys, mmap
-from bup import options, _hashsplit
+import sys
+from bup import options, _helpers
 from bup.helpers import *
 
 optspec = """
 bup random [-S seed] <numbytes>
 --
-S,seed=   optional random number seed (default 1)
+S,seed=   optional random number seed [1]
 f,force   print random data to stdout even if it's a tty
 """
 o = options.Options('bup random', optspec)
@@ -17,9 +17,11 @@ if len(extra) != 1:
 
 total = parse_num(extra[0])
 
+handle_ctrl_c()
+
 if opt.force or (not os.isatty(1) and
                  not atoi(os.environ.get('BUP_FORCE_TTY')) & 1):
-    _hashsplit.write_random(sys.stdout.fileno(), total, opt.seed or 0)
+    _helpers.write_random(sys.stdout.fileno(), total, opt.seed)
 else:
     log('error: not writing binary data to a terminal. Use -f to force.\n')
     sys.exit(1)
