@@ -1,6 +1,7 @@
 #!/usr/bin/env python
-import sys, os, glob
-from bup import options, _version
+import sys
+from bup import options
+from bup.helpers import *
 
 optspec = """
 bup version [--date|--commit|--tag]
@@ -12,24 +13,14 @@ tag     display the tag name of this version.  If no tag is available, display t
 o = options.Options('bup version', optspec)
 (opt, flags, extra) = o.parse(sys.argv[1:])
 
-def autoname(names):
-    names = names.strip()
-    assert(names[0] == '(')
-    assert(names[-1] == ')')
-    names = names[1:-1]
-    l = [n.strip() for n in names.split(',')]
-    for n in l:
-        if n.startswith('tag: bup-'):
-            return n[9:]
-
 
 total = (opt.date or 0) + (opt.commit or 0) + (opt.tag or 0)
 if total > 1:
     o.fatal('at most one option expected')
 
 if opt.date:
-    print _version.DATE.split(' ')[0]
+    print version_date()
 elif opt.commit:
-    print _version.COMMIT
+    print version_commit()
 else:
-    print autoname(_version.NAMES) or 'unknown-%s' % _version.COMMIT[:7]
+    print version_tag()
